"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
/*
 * Copyright (c) 2021, salesforce.com, inc.
 * All rights reserved.
 * Licensed under the BSD 3-Clause license.
 * For full license text, see LICENSE.txt file in the repo root or https://opensource.org/licenses/BSD-3-Clause
 */
const convert_1 = require("../convert");
const diagnosticUtil_1 = require("./diagnosticUtil");
const types_1 = require("./types");
const metadataTransfer_1 = require("./metadataTransfer");
class MetadataApiDeploy extends metadataTransfer_1.MetadataTransfer {
    constructor(options) {
        super(options);
        this.options = Object.assign({}, MetadataApiDeploy.DEFAULT_OPTIONS, options);
    }
    pre() {
        return __awaiter(this, void 0, void 0, function* () {
            const converter = new convert_1.MetadataConverter();
            const { zipBuffer } = yield converter.convert(Array.from(this.components.getSourceComponents()), 'metadata', { type: 'zip' });
            const connection = yield this.getConnection();
            const result = yield connection.metadata.deploy(zipBuffer, this.options.apiOptions);
            this.deployId = result.id;
            return result;
        });
    }
    checkStatus(id) {
        return __awaiter(this, void 0, void 0, function* () {
            const connection = yield this.getConnection();
            // Recasting to use the project's DeployResult type
            return connection.metadata.checkDeployStatus(id, true);
        });
    }
    post(result) {
        return __awaiter(this, void 0, void 0, function* () {
            const diagnosticUtil = new diagnosticUtil_1.DiagnosticUtil('metadata');
            const componentDeploymentMap = new Map();
            const deployResult = {
                id: result.id,
                status: result.status,
                success: result.success,
            };
            for (const component of this.components.getSourceComponents()) {
                componentDeploymentMap.set(`${component.type.name}:${component.fullName}`, {
                    status: types_1.ComponentStatus.Unchanged,
                    component,
                    diagnostics: [],
                });
            }
            for (let message of this.getDeployMessages(result)) {
                message = this.sanitizeDeployMessage(message);
                const componentKey = `${message.componentType}:${message.fullName}`;
                const componentDeployment = componentDeploymentMap.get(componentKey);
                if (componentDeployment) {
                    if (message.created === 'true') {
                        componentDeployment.status = types_1.ComponentStatus.Created;
                    }
                    else if (message.changed === 'true') {
                        componentDeployment.status = types_1.ComponentStatus.Changed;
                    }
                    else if (message.deleted === 'true') {
                        componentDeployment.status = types_1.ComponentStatus.Deleted;
                    }
                    else if (message.success === 'false') {
                        componentDeployment.status = types_1.ComponentStatus.Failed;
                    }
                    else {
                        componentDeployment.status = types_1.ComponentStatus.Unchanged;
                    }
                    if (message.problem) {
                        diagnosticUtil.setDeployDiagnostic(componentDeployment, message);
                    }
                }
            }
            deployResult.components = Array.from(componentDeploymentMap.values());
            return deployResult;
        });
    }
    doCancel() {
        return __awaiter(this, void 0, void 0, function* () {
            let done = true;
            if (this.deployId) {
                const connection = yield this.getConnection();
                // @ts-ignore _invoke is private on the jsforce metadata object, and cancelDeploy is not an exposed method
                done = connection.metadata._invoke('cancelDeploy', { id: this.deployId }).done;
            }
            return done;
        });
    }
    getDeployMessages(result) {
        const messages = [];
        const { componentSuccesses, componentFailures } = result.details;
        if (componentSuccesses) {
            if (Array.isArray(componentSuccesses)) {
                messages.push(...componentSuccesses);
            }
            else {
                messages.push(componentSuccesses);
            }
        }
        if (componentFailures) {
            if (Array.isArray(componentFailures)) {
                messages.push(...componentFailures);
            }
            else {
                messages.push(componentFailures);
            }
        }
        return messages;
    }
    /**
     * Fix any issues with the deploy message returned by the api.
     * TODO: remove as fixes are made in the api.
     */
    sanitizeDeployMessage(message) {
        // lwc doesn't properly use the fullname property in the api.
        message.fullName = message.fullName.replace(/markup:\/\/c:/, '');
        return message;
    }
}
exports.MetadataApiDeploy = MetadataApiDeploy;
MetadataApiDeploy.DEFAULT_OPTIONS = {
    apiOptions: {
        rollbackOnError: true,
        ignoreWarnings: false,
        checkOnly: false,
        singlePackage: true,
    },
};
//# sourceMappingURL=metadataApiDeploy.js.map